@@ -20,21 +20,34 @@ namespace Beagle.Filters {
 
 		public FilterPdf ()
 		{
-			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/pdf"));
 			SnippetMode = true;
 		}
 
+		protected override void RegisterSupportedTypes ()
+		{
+			AddSupportedFlavor (FilterFlavor.NewFromMimeType ("application/pdf"));
+		}
+
 		// FIXME: we should have a reasonable failure mode if pdftotext is
 		// not installed.
 
+		SafeProcess pc = null;
+		StreamReader pout = null;
+
 		protected override void DoPullProperties ()
 		{
 			// create new external process
-			SafeProcess pc = new SafeProcess ();
+			pc = new SafeProcess ();
 			pc.Arguments = new string [] { "pdfinfo", FileInfo.FullName };
 			pc.RedirectStandardOutput = true;
 			pc.RedirectStandardError = true;
 
+			// Runs inside the child process after form() but before exec()
+			pc.ChildProcessSetup += delegate {
+				// Let pdfinfo run for 10 CPU seconds, max.
+				SystemPriorities.SetResourceLimit (SystemPriorities.Resource.Cpu, 10);
+			};
+
 			try {
 				pc.Start ();
 			} catch (SafeProcessException e) {
@@ -44,7 +57,7 @@ namespace Beagle.Filters {
 			}
 
 			// add pdfinfo's output to pool
-			StreamReader pout = new StreamReader (pc.StandardOutput);
+			pout = new StreamReader (pc.StandardOutput);
 			string str = null;
 			string[] tokens = null;
 			string strMetaTag = null;
@@ -95,44 +108,82 @@ namespace Beagle.Filters {
 			pc.Close ();
 		}
 		
-		protected override void DoPull ()
+		bool pull_started = false;
+
+		private bool InitDoPull ()
 		{
 			// create new external process
-			SafeProcess pc = new SafeProcess ();
-			pc.Arguments = new string [] { "pdftotext", "-q", "-nopgbrk", "-enc", "UTF-8", FileInfo.FullName, "-" };
+			pc = new SafeProcess ();
+			pc.Arguments = new string [] { "pdftotext", "-nopgbrk", "-enc", "UTF-8", FileInfo.FullName, "-" };
 			pc.RedirectStandardOutput = true;
 			pc.RedirectStandardError = true;
 
+			// Runs inside the child process after form() but before exec()
+			pc.ChildProcessSetup += delegate {
+				// Let pdftotext run for 90 CPU seconds, max.
+				SystemPriorities.SetResourceLimit (SystemPriorities.Resource.Cpu, 90);
+			};
+
 			try {
 				pc.Start ();
 			} catch (SafeProcessException e) {
 				Log.Warn (e.Message);
 				Error ();
-				return;
+				return false;
 			}
 
 			// add pdftotext's output to pool
-			StreamReader pout = new StreamReader (pc.StandardOutput);
+			pout = new StreamReader (pc.StandardOutput);
+			pull_started = true;
 
-			// FIXME:  I don't think this is really required
-			// Line by line parsing, however, we have to make
-			// sure, that "pdftotext" doesn't output any "New-lines".
-			string str;
-			while ((str = pout.ReadLine()) != null) {
-				AppendText (str);
-				AppendStructuralBreak ();
-				if (! AllowMoreWords ())
-					break;
+			return true;
+		}
+
+		protected override void DoPull ()
+		{
+			// InitDoPull() calls Error() if it fails
+			if (! pull_started && ! InitDoPull ())
+				return;
+
+			int n = 0;
+
+			// Using internal information: Lucene currently asks for char[2048] data
+			while (n <= 2048) {
+
+				// FIXME:  I don't think this is really required
+				// Line by line parsing, however, we have to make
+				// sure, that "pdftotext" doesn't output any "New-lines".
+				string str = pout.ReadLine ();
+				if (str == null) {
+					Finished ();
+					return;
+				} else {
+					AppendLine (str);
+					AppendStructuralBreak ();
+					// If we have added 2048 chars, stop
+					// DoPull is called repeatedly till the buffer is full,
+					// so stop after the buffer is full (and possibly overflown)
+					// to reduce number of function calls
+					n += str.Length;
+					n ++; // for the structural break
+				}
 			}
-			pout.Close ();
+		}
 
+		override protected void DoClose ()
+		{
+			if (! pull_started)
+				return;
+
+			pout.Close ();
 			pout = new StreamReader (pc.StandardError);
+
+			string str;
 			while ((str = pout.ReadLine ()) != null)
 				Log.Warn ("pdftotext [{0}]: {1}", Uri, str);
 
 			pout.Close ();
 			pc.Close ();
-			Finished ();
 		}
 	}
 }
