@@ -298,29 +298,12 @@ namespace Beagle.Util {
 		[ConfigSection (Name="searching")]
 		public class SearchingConfig : Section {
 			
-			private bool autostart = true;
-			public bool Autostart {
-				get { return autostart; }
-				set { autostart = value; }
-			}
-			
 			private KeyBinding show_search_window_binding = new KeyBinding ("F12");
 			public KeyBinding ShowSearchWindowBinding {
 				get { return show_search_window_binding; }
 				set { show_search_window_binding = value; }
 			}
 
-			private int max_displayed = 5;
-			public int MaxDisplayed {
-				get { return max_displayed; }
-				set {
-					if (value <= 0)
-						max_displayed = 1;
-					else
-						max_displayed = value;
-				}
-			}
-
 			// BeagleSearch window position and dimension
 			// stored as percentage of screen co-ordinates
 			// to deal with change of resolution problem - hints from tberman
@@ -356,6 +339,12 @@ namespace Beagle.Util {
 				set { search_history = value; }
 			}
 
+			private bool beagle_search_auto_search = true;
+			public bool BeagleSearchAutoSearch {
+				get { return beagle_search_auto_search; }
+				set { beagle_search_auto_search = value; }
+			}
+
 		}
 
 		[ConfigSection (Name="daemon")]
@@ -559,7 +548,7 @@ namespace Beagle.Util {
 				ExcludeType type;
 				try {
 					type = (ExcludeType) Enum.Parse (typeof (ExcludeType), args [0], true);
-				} catch (Exception e) {
+				} catch {
 					output = String.Format("Invalid type '{0}'. Valid types: Path, Pattern, MailFolder", args [0]);
 					return false;
 				}
@@ -575,7 +564,7 @@ namespace Beagle.Util {
 				ExcludeType type;
 				try {
 					type = (ExcludeType) Enum.Parse (typeof (ExcludeType), args [0], true);
-				} catch (Exception e) {
+				} catch {
 					output = String.Format("Invalid type '{0}'. Valid types: Path, Pattern, MailFolder", args [0]);
 					return false;
 				}
