@@ -281,6 +281,8 @@ namespace Beagle.Util {
 
 					if (Reschedule) {
 						++count;
+						if (Debug)
+							Log.Debug ("Rescheduling task {0}", Tag);
 						scheduler.Add (this); // re-add ourselves
 					} else {
 						DecrementAllTaskGroups ();
@@ -521,6 +523,8 @@ namespace Beagle.Util {
 			exercise = Environment.GetEnvironmentVariable ("BEAGLE_EXERCISE_THE_DOG");
 
 			if (exercise != null) {
+				Log.Always ("BEAGLE_EXERCISE_THE_DOG is set.");
+
 				if (exercise.Length > 2 && exercise [0] == 't')
 					global_delay = Double.Parse (exercise.Substring (1));
 				else
@@ -577,10 +581,9 @@ namespace Beagle.Util {
 					return;
 
 				if (Debug) {
-					Logger.Log.Debug ("Adding task");
-					Logger.Log.Debug ("Tag: {0}", task.Tag);
+					Logger.Log.Debug ("Adding task {0}", task.Tag);
 					if (task.Description != null)
-						Logger.Log.Debug ("Desc: {0}", task.Description);
+						Logger.Log.Debug ("  Desc: {0}", task.Description);
 				}
 
 				if (task.Priority == Priority.Shutdown)
@@ -899,8 +902,10 @@ namespace Beagle.Util {
 					// If we still need to wait a bit longer, wait for the appropriate
 					// amount of time and then re-start our while loop.
 					if (delay > 0.001) {
-						status_str = "Waiting for next task.";
-						Monitor.Wait (big_lock, TimeSpanFromSeconds (delay));
+						TimeSpan span = TimeSpanFromSeconds (delay);
+
+						status_str = String.Format ("Waiting for next task at {0}", now + span);
+						Monitor.Wait (big_lock, span);
 						executed_task_count = 0;
 						continue;
 					}
@@ -914,7 +919,7 @@ namespace Beagle.Util {
 
 					if (next_task.Collector == null) {
 
-							to_be_executed.Add (next_task);
+						to_be_executed.Add (next_task);
 
 					} else {
 
