@@ -39,21 +39,35 @@ class ExtractContentTool {
 	static bool tokenize = false;
 	static bool show_children = false;
 	static string mime_type = null;
+	static bool continue_last = false;
 
 	// FIXME: We don't display structural breaks
-	static void DisplayContent (string line)
+	static void DisplayContent (char[] buffer, int length)
 	{
 		if (tokenize) {
-			
+			if (continue_last && buffer [0] == ' ')
+				Console.WriteLine ();
+
+			char last_char = buffer [length - 1];
+			continue_last = (last_char != '\n' &&
+					      last_char != '\t' &&
+					      last_char != ' ');
+
+			string line = new string (buffer, 0, length);
 			string [] parts = line.Split (' ');
-			for (int i = 0; i < parts.Length; ++i) {
+			for (int i = 0; i < parts.Length - 1; ++i) {
 				string part = parts [i].Trim ();
-				if (part != "")
+				if (part != String.Empty)
 					Console.WriteLine ("{0}", part);
 			}
 
+			string last = parts [parts.Length - 1];
+			last = last.Trim ();
+			if (last != String.Empty)
+				Console.Write ("{0}{1}", last, (continue_last ? "" : "\n"));
+
 		} else {
-			Console.WriteLine (line);
+			Console.Write (buffer, 0, length);
 		}
 	}
 
@@ -70,20 +84,24 @@ class ExtractContentTool {
 
 		Console.WriteLine ("Filename: " + indexable.Uri);
 
+		Stopwatch watch = new Stopwatch ();
+
 		Filter filter;
 
+		watch.Start ();
 		if (! FilterFactory.FilterIndexable (indexable, out filter)) {
 			Console.WriteLine ("No filter for {0}", indexable.MimeType);
 			indexable.Cleanup ();
 			return;
 		}
+		watch.Stop ();
 
-		Console.WriteLine ("Filter: {0}", filter);
+		Console.WriteLine ("Filter: {0} (determined in {1})", filter, watch);
 		Console.WriteLine ("MimeType: {0}", filter.MimeType);
 		Console.WriteLine ();
 
 		if (filter.ChildIndexables != null && filter.ChildIndexables.Count > 0) {
-			Console.WriteLine ("Child indexables:");
+			Console.WriteLine ("Child indexables ({0}):", filter.ChildIndexables.Count);
 
 			foreach (Indexable i in filter.ChildIndexables)
 				Console.WriteLine ("  {0}", i.Uri);
@@ -98,61 +116,83 @@ class ExtractContentTool {
 		bool first = true;
 
 		Console.WriteLine ("Properties:");
-		Console.WriteLine ("  Timestamp = {0}", indexable.Timestamp);
+
+		if (indexable.ValidTimestamp)
+			Console.WriteLine ("  Timestamp = {0}", DateTimeUtil.ToString (indexable.Timestamp));
+
 		foreach (Beagle.Property prop in prop_array) {
 			Console.WriteLine ("  {0} = {1}", prop.Key, prop.Value);
 		}
+
 		Console.WriteLine ();
 
 		if (indexable.NoContent)
 			return;
 
+		watch.Reset ();
+		watch.Start ();
+
 		TextReader reader;
 
+		char[] buffer = new char [2048];
 		reader = indexable.GetTextReader ();
 		if (reader != null) {
-			string line;
-			first = true;
-			while ((line = reader.ReadLine ()) != null) {
-				if (first) {
-					Console.WriteLine ("Content:");
+			Console.WriteLine ("Content:");
+			while (true) {
+				int l = reader.Read (buffer, 0, 2048);
+				if (l <= 0)
+					break;
+				if (first)
 					first = false;
-				}
-				DisplayContent (line);
+				DisplayContent (buffer, l);
 			}
 			reader.Close ();
 
 			if (first)
 				Console.WriteLine ("(no content)");
 			else
-				Console.WriteLine ();
+				Console.WriteLine ('\n');
 		}
 			
 		reader = indexable.GetHotTextReader ();
 		if (reader != null) {
-			string line;
+			Console.WriteLine ("HotContent:");
 			first = true;
-			while ((line = reader.ReadLine ()) != null) {
-				if (first) {
-					Console.WriteLine ("HotContent:");
+			while (true) {
+				int l = reader.Read (buffer, 0, 2048);
+				if (l <= 0)
+					break;
+				if (first)
 					first = false;
-				}
-				DisplayContent (line);
+				DisplayContent (buffer, l);
 			}
 			reader.Close ();
 
 			if (first)
 				Console.WriteLine ("(no hot content)");
 			else
-				Console.WriteLine ();
+				Console.WriteLine ('\n');
 		}
 
+		watch.Stop ();
+
+		Console.WriteLine ();
+		Console.WriteLine ("Text extracted in {0}", watch);
+
 		Stream stream = indexable.GetBinaryStream ();
 		if (stream != null)
 			stream.Close ();
 
-		if (show_children && filter.ChildIndexables != null) {
+		// Clean up any temporary files associated with filtering this indexable.
+		indexable.Cleanup ();
+
+		if (filter.ChildIndexables != null) {
 			foreach (Indexable i in filter.ChildIndexables) {
+				if (! show_children) {
+					i.Cleanup ();
+					continue;
+				}
+
 				i.StoreStream ();
 				Display (i);
 			}
