@@ -17,20 +17,25 @@
  *  along with this program; if not, write to the Free Software
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- *  $Id: ephy-beagle.c,v 1.1 2004/05/19 02:38:53 trow Exp $
+ *  $Id: ephy-beagle.c 1306 2005-05-28 14:10:55Z chpe $
  */
 
 /*
   This is all copied from Dashboard's Epiphany Extension.
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include "ephy-beagle-extension.h"
 
 #include <gmodule.h>
 
+G_MODULE_EXPORT GType register_module (GTypeModule *module);
+
 G_MODULE_EXPORT GType
 register_module (GTypeModule *module)
 {
 	return ephy_beagle_extension_register_type (module);
 }
-
